package category

import (
	"context"
	"github.com/iVampireSP/pkg/page"
	"leafdev.top/Ecosystem/recommender/internal/entity"
	"leafdev.top/Ecosystem/recommender/internal/schema"
)

func (s *Service) ListCategories(c context.Context, pagedResult *page.PagedResult[*entity.Category], application *entity.Application) error {
	var q = s.dao.WithContext(c).Category

	if application != nil {
		q = q.Preload(s.dao.Category.Application).Where(s.dao.Category.ApplicationId.Eq(application.Id.Uint()))
	}

	posts, count, err := q.FindByPage(pagedResult.Offset(), pagedResult.PageSize)
	if err != nil {
		return err
	}

	pagedResult.Data = posts
	pagedResult.TotalCount = count

	return nil
}

func (s *Service) CreateCategory(c context.Context, category *entity.Category, applicationEntity *entity.Application) error {
	category.ApplicationId = applicationEntity.Id

	return s.dao.WithContext(c).Category.Create(category)
}

func (s *Service) DeleteCategory(c context.Context, category *entity.Category) error {
	_, err := s.dao.WithContext(c).Category.Delete(category)

	return err
}

func (s *Service) GetCategoryById(c context.Context, categoryId schema.EntityId) (*entity.Category, error) {
	return s.dao.WithContext(c).Category.Where(s.dao.Category.Id.Eq(categoryId.Uint())).First()
}
