package post

import (
	"context"
	"leafdev.top/Ecosystem/recommender/internal/entity"
)

func (s *Service) GetTag(c context.Context, name string, applicationEntity *entity.Application) (*entity.Tag, error) {
	var tmq = s.dao.WithContext(c).TagMapping.Where(s.dao.TagMapping.Name.Eq(name)).
		Where(s.dao.TagMapping.ApplicationId.Eq(applicationEntity.Id.Uint()))
	tmqCount, err := tmq.Count()
	if err != nil {
		return nil, err
	}

	if tmqCount > 0 {
		r, err := tmq.First()

		if err != nil {
			return nil, err
		}

		return r.Tag, nil
	}

	return s.dao.WithContext(c).Tag.Where(s.dao.Tag.Name.Eq(name)).
		Where(s.dao.Tag.ApplicationId.Eq(applicationEntity.Id.Uint())).
		FirstOrCreate()
}

func (s *Service) HasBindTag(c context.Context, post *entity.Post, tagName string) (bool, error) {
	tag, err := s.GetTag(c, tagName, post.Application)
	if err != nil {
		return false, err
	}

	count, err := s.dao.WithContext(c).PostTag.Where(s.dao.PostTag.PostId.Eq(post.Id.Uint())).
		Where(s.dao.PostTag.TagId.Eq(tag.Id.Uint())).Count()
	if err != nil {
		return false, err
	}

	return count > 0, nil
}

func (s *Service) BindTag(c context.Context, post *entity.Post, tagName string) error {
	tag, err := s.GetTag(c, tagName, post.Application)
	if err != nil {
		return err
	}

	bind, err := s.HasBindTag(c, post, tag.Name)
	if err != nil {
		return err
	}

	if !bind {
		err = s.dao.WithContext(c).PostTag.Create(&entity.PostTag{
			PostId: &post.Id,
			TagId:  &tag.Id,
		})
	}

	return nil
}

func (s *Service) MarkAsProcessed(c context.Context, post *entity.Post) error {
	_, err := s.dao.WithContext(c).Post.Where(s.dao.Post.Id.Eq(post.Id.Uint())).Update(s.dao.Post.Processed, true)

	return err
}
