package user

import (
	"context"
	"leafdev.top/Ecosystem/recommender/internal/entity"
)

func (s *Service) HasBindTag(c context.Context, externalUserEntity *entity.ExternalUser, applicationEntity *entity.Application, tagEntity *entity.Tag) (bool, error) {
	count, err := s.dao.WithContext(c).UserTagScore.
		Where(s.dao.UserTagScore.ExternalUserId.Eq(externalUserEntity.Id.Uint())).
		Where(s.dao.UserTagScore.TagId.Eq(tagEntity.Id.Uint())).
		Where(s.dao.UserTagScore.ApplicationId.Eq(applicationEntity.Id.Uint())).Count()
	if err != nil {
		return false, err
	}

	return count > 0, nil
}

func (s *Service) BindTags(c context.Context, externalUserEntity *entity.ExternalUser, applicationEntity *entity.Application, tags []*entity.Tag) error {
	// 检测用户是否绑定 Tag，如果绑定则更新，否则创建
	for _, tag := range tags {
		var uts *entity.UserTagScore

		hasBind, err := s.HasBindTag(c, externalUserEntity, applicationEntity, tag)
		if err != nil {
			return err
		}
		if hasBind {
			// score + 1
			uts, err = s.dao.WithContext(c).UserTagScore.
				Where(s.dao.UserTagScore.ExternalUserId.Eq(externalUserEntity.Id.Uint())).
				Where(s.dao.UserTagScore.TagId.Eq(tag.Id.Uint())).
				Where(s.dao.UserTagScore.ApplicationId.Eq(applicationEntity.Id.Uint())).First()
			if err != nil {
				return err
			}

			uts.Score += 1

			err = s.dao.WithContext(c).UserTagScore.Save(uts)
			if err != nil {
				return err
			}

			continue
		}

		uts = &entity.UserTagScore{
			ApplicationId:  applicationEntity.Id,
			TagId:          tag.Id,
			ExternalUserId: externalUserEntity.Id,
			Score:          1,
		}

		err = s.dao.WithContext(c).UserTagScore.Create(uts)
		if err != nil {
			return err
		}
	}

	return nil
}

func (s *Service) RemoveTags(c context.Context, externalUserEntity *entity.ExternalUser, applicationEntity *entity.Application, tags []*entity.Tag) error {
	for _, tag := range tags {
		var uts *entity.UserTagScore

		hasBind, err := s.HasBindTag(c, externalUserEntity, applicationEntity, tag)
		if err != nil {
			return err
		}
		if hasBind {
			// score - 1
			uts, err = s.dao.WithContext(c).UserTagScore.
				Where(s.dao.UserTagScore.ExternalUserId.Eq(externalUserEntity.Id.Uint())).
				Where(s.dao.UserTagScore.TagId.Eq(tag.Id.Uint())).
				Where(s.dao.UserTagScore.ApplicationId.Eq(applicationEntity.Id.Uint())).First()
			if err != nil {
				return err
			}

			//if uts.Score-1 < 0 {
			//	continue
			//}

			uts.Score -= 1

			_, err = s.dao.WithContext(c).UserTagScore.
				Where(s.dao.UserTagScore.ExternalUserId.Eq(externalUserEntity.Id.Uint())).
				Where(s.dao.UserTagScore.TagId.Eq(tag.Id.Uint())).
				Where(s.dao.UserTagScore.ApplicationId.Eq(applicationEntity.Id.Uint())).Update(s.dao.UserTagScore.Score, uts.Score)
			if err != nil {
				return err
			}

			continue
		} else {
			uts = &entity.UserTagScore{
				ApplicationId:  applicationEntity.Id,
				TagId:          tag.Id,
				ExternalUserId: externalUserEntity.Id,
				Score:          -1,
			}

			err = s.dao.WithContext(c).UserTagScore.Create(uts)
			if err != nil {
				return err
			}
		}
	}

	return nil
}
